@@ -33,10 +33,8 @@ static void
 html_box_embedded_select_relayout (HtmlBox *box, HtmlRelayout *relayout) 
 {
 	DomHTMLSelectElement *select_node = DOM_HTML_SELECT_ELEMENT (box->dom_node);
-	GtkTreeModel *model = dom_html_select_element_get_tree_model (select_node);
 	HtmlStyle *style = HTML_BOX_GET_STYLE (box);
 	GtkWidget *widget = HTML_BOX_EMBEDDED (box)->widget;
-	HtmlBoxEmbeddedSelect *select = HTML_BOX_EMBEDDED_SELECT (box);
 
 	if (dom_HTMLSelectElement__get_multiple (select_node) == FALSE &&
 	    dom_HTMLSelectElement__get_size (select_node) == 1) {
@@ -86,7 +84,6 @@ static void
 update_treeview_selection (GtkWidget *widget, GtkRequisition *req,
 			   gpointer data)
 {
-	DomHTMLSelectElement *select_node;
 	GtkTreeModel *model;
 
 	model = GTK_TREE_MODEL (data);
@@ -155,6 +152,7 @@ combo_selection_changed (GtkComboBox *widget, gpointer data)
 	g_free (value);
 }
 
+static void
 row_changed_callback (GtkTreeModel *model, GtkTreePath *path,
 		      GtkTreeIter *iter, gpointer data)
 {
