@@ -24,6 +24,7 @@
 #include "layout/htmlbox.h"
 #include "layout/htmlboxtext.h"
 #include "layout/html/htmlboxform.h"
+#include "layout/html/htmlboximage.h"
 #include "layout/htmlboxinline.h"
 #include "layout/htmlboxtablerow.h"
 #include "gtkhtmlcontext.h"
@@ -37,16 +38,12 @@ html_box_get_containing_block (HtmlBox *box)
 
 	while (box) {
 		/* This is three times less expensive then HTML_IS_BOX_BLOCK  */
-#if 1
 		HtmlStyle *style = HTML_BOX_GET_STYLE(box);
 		if (style->display == HTML_DISPLAY_BLOCK ||
 		    style->display == HTML_DISPLAY_LIST_ITEM ||
 		    style->display == HTML_DISPLAY_TABLE_CELL)
 			return box;
-else
-		if (HTML_IS_BOX_BLOCK (box))
-			return box;
-#endif
+
 		box = box->parent;
 	}
 
@@ -154,17 +151,13 @@ simple_margin (HtmlStyle *style)
 static gboolean
 need_containing_width (HtmlBox *box, gint width)
 {
-	HtmlStyle *style;
-
 	if (width > 0)
 		return FALSE;
 
-	style = HTML_BOX_GET_STYLE (box);
+	if (html_box_get_containing_block (box))
+		return TRUE;
 
-	if (simple_margin (style))
-		return FALSE;
-
-	return TRUE;
+	return FALSE;
 }
 
 gint
@@ -311,7 +304,7 @@ html_box_left_margin (HtmlBox *box, gint width)
 	else if (style->surround->margin.left.type != HTML_LENGTH_AUTO &&
 		 style->surround->margin.right.type != HTML_LENGTH_AUTO) {
 
-		if (HTML_BOX_GET_STYLE (box->parent)->inherited->direction == HTML_DIRECTION_RTL) {
+		if (HTML_BOX_GET_STYLE (html_box_get_containing_block (box))->inherited->direction == HTML_DIRECTION_RTL) {
 
 			return width - html_length_get_value (&style->box->width, width) -
 				html_box_left_padding (box, width) - html_box_right_padding (box, width) -
@@ -351,7 +344,7 @@ html_box_right_margin (HtmlBox *box, gint width)
 	else if (style->surround->margin.left.type != HTML_LENGTH_AUTO &&
 		 style->surround->margin.right.type != HTML_LENGTH_AUTO) {
 
-		if (HTML_BOX_GET_STYLE (box->parent)->inherited->direction == HTML_DIRECTION_LTR) {
+		if (HTML_BOX_GET_STYLE (html_box_get_containing_block (box))->inherited->direction == HTML_DIRECTION_LTR) {
 
 			return width - html_length_get_value (&style->box->width, width) -
 				html_box_left_padding (box, width) - html_box_right_padding (box, width) -
@@ -456,6 +449,9 @@ html_box_finalize (GObject *object)
 	if (box->style)
 		html_style_unref (box->style);
 
+	if (box->dom_node)
+		g_object_remove_weak_pointer (G_OBJECT (box->dom_node), (gpointer *) &(box->dom_node));
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 	
 }
@@ -818,35 +814,38 @@ void
 html_box_check_min_max_width_height (HtmlBox *self, gint *boxwidth, gint *boxheight) 
 {
 	int tmp;
+	gint old_width = *boxwidth;
+	gint old_height = *boxheight;
+	HtmlBox *parent = html_box_get_containing_block (self);
 
-	if (self->parent) {
+	if (parent) {
 		if (HTML_BOX_GET_STYLE (self)->box->min_width.type != HTML_LENGTH_AUTO) {
-			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->min_width, self->parent->width -
-						     html_box_horizontal_mbp_sum (self->parent) - html_box_horizontal_mbp_sum (self));
+			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->min_width, parent->width -
+						     html_box_horizontal_mbp_sum (parent) - html_box_horizontal_mbp_sum (self));
 			if (*boxwidth < tmp)
 				*boxwidth = tmp;		
 
 		}
 		
 		if (HTML_BOX_GET_STYLE (self)->box->max_width.type != HTML_LENGTH_AUTO) {
-			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_width, self->parent->width -
-						     html_box_horizontal_mbp_sum (self->parent) - html_box_horizontal_mbp_sum (self));
+			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_width, parent->width -
+						     html_box_horizontal_mbp_sum (parent) - html_box_horizontal_mbp_sum (self));
 			if (*boxwidth > tmp)
 				*boxwidth = tmp;		
 
 		}
 		
 		if (HTML_BOX_GET_STYLE (self)->box->min_height.type != HTML_LENGTH_AUTO) {
-			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->min_height, self->parent->height -
-						     html_box_horizontal_mbp_sum (self->parent) - html_box_horizontal_mbp_sum (self));
+			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->min_height, parent->height -
+						     html_box_horizontal_mbp_sum (parent) - html_box_horizontal_mbp_sum (self));
 			if (*boxheight < tmp)
 				*boxheight = tmp;		
 
 		}
 		
 		if (HTML_BOX_GET_STYLE (self)->box->max_height.type != HTML_LENGTH_AUTO) {
-			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_height, self->parent->height -
-						     html_box_horizontal_mbp_sum (self->parent) - html_box_horizontal_mbp_sum (self));
+			tmp = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_height, parent->height -
+						     html_box_horizontal_mbp_sum (parent) - html_box_horizontal_mbp_sum (self));
 			if (*boxheight > tmp)
 				*boxheight = tmp;		
 
@@ -869,6 +868,26 @@ html_box_check_min_max_width_height (HtmlBox *self, gint *boxwidth, gint *boxhei
 			if (*boxheight > html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_height, 0))
 				*boxheight = html_length_get_value (&HTML_BOX_GET_STYLE (self)->box->max_height, 0);
 	}
+	
+	/* Maintain aspect ratio if it's an image */
+	if (HTML_IS_BOX_IMAGE (self)) {
+		if ((*boxwidth > old_width) && (*boxheight >= old_height)) {
+			*boxheight = *boxheight * (gdouble)(*boxwidth / (gdouble)old_width);
+			return;
+		}
+		else if ((*boxheight > old_height) && (*boxwidth >= old_width)) {
+			*boxwidth = *boxwidth * (gdouble)(*boxheight / (gdouble)old_height);
+			return;
+		}
+		else if ((*boxwidth < old_width) && (*boxheight <= old_height)) {
+			*boxheight = *boxheight * (gdouble)(*boxwidth / (gdouble)old_width);
+			return;
+		}
+		else if ((*boxheight < old_height) && (*boxwidth <= old_width)) {
+			*boxwidth = *boxwidth * (gdouble)(*boxheight / (gdouble)old_height);
+			return;
+		}
+	}
 }
 
 static void
