@@ -68,8 +68,10 @@ html_box_table_row_update_spaninfo (HtmlBoxTableRow *row, gint *spaninfo)
 		/*
 		 * This is to allow <form> tags in the middle of <tr> and <td>
 		 */
-		if (HTML_IS_BOX_FORM (box))
-			cols += html_box_table_row_update_spaninfo (HTML_BOX_TABLE_ROW (box), &spaninfo[cols]);
+		if (HTML_IS_BOX_FORM (box)) {
+			if (HTML_IS_BOX_TABLE_ROW (box))
+				cols += html_box_table_row_update_spaninfo (HTML_BOX_TABLE_ROW (box), &spaninfo[cols]);
+		}
 
 		if (!HTML_IS_BOX_TABLE_CELL (box)) {
 			box = box->next;
